@@ -37,7 +37,7 @@ gboolean config_output = FALSE, config_remember = TRUE, config_errorlist = FALSE
 #ifndef MAEMO
 gboolean config_switchbutton = FALSE, config_trayicon = FALSE;
 #endif
-GTimeVal startime, pausedtime;
+gint64 startime, pausedtime;
 gchar config_command[MAXPATH], *grsync_dir, *rsync_help = NULL, *rsync_man = NULL;
 FILE *log_file = NULL;
 glong scroll_previous_run;
@@ -531,15 +531,15 @@ void show_browse_dest(GtkButton *button, gpointer user_data) {
 
 
 void set_global_progress(gdouble fraction) {
-	GTimeVal curtime;
+	gint64 curtime;
 	glong elapsed, remaining;
 	static glong lastupdate = 0;
 	gchar tmps[50];
 	GtkProgressBar* progr = (GtkProgressBar*) gtk_builder_get_object(builder, "progress_global");
 
-	g_get_current_time(&curtime);
-	if (fraction > (gdouble)0 && fraction < (gdouble)1 && curtime.tv_sec == lastupdate) return;
-	lastupdate = curtime.tv_sec;
+	curtime = g_get_monotonic_time() / 1000000;
+	if (fraction > (gdouble)0 && fraction < (gdouble)1 && curtime == lastupdate) return;
+	lastupdate = curtime;
 
 	#ifdef HAVE_UNITY
 		UnityLauncherEntry *launcher_entry = unity_launcher_entry_get_for_desktop_id("grsync.desktop");
@@ -548,7 +548,7 @@ void set_global_progress(gdouble fraction) {
  	#endif
 
 	gtk_progress_bar_set_fraction(progr, fraction);
-	elapsed = curtime.tv_sec - startime.tv_sec;
+	elapsed = curtime - startime;
 	remaining = elapsed / fraction - elapsed;
 	if (fraction) snprintf(tmps, 49, _("%.f%% (%li:%02li elapsed, %li:%02li remaining)"), round(fraction * 100), elapsed / 60, elapsed % 60, remaining / 60, remaining % 60);
 	else strcpy(tmps, "0%");
@@ -558,14 +558,14 @@ void set_global_progress(gdouble fraction) {
 
 
 void set_file_progress(gdouble fraction) {
-	GTimeVal curtime;
+	gint64 curtime;
 	static glong lastupdate = 0;
 	gchar tmps[50];
 	GtkProgressBar* progr = (GtkProgressBar*) gtk_builder_get_object(builder, "progress_file");
 
-	g_get_current_time(&curtime);
-	if (fraction < (gdouble)1 && curtime.tv_sec == lastupdate) return;
-	lastupdate = curtime.tv_sec;
+	curtime = g_get_monotonic_time() / 1000000;
+	if (fraction < (gdouble)1 && curtime == lastupdate) return;
+	lastupdate = curtime;
 
 	gtk_progress_bar_set_fraction(progr, fraction);
 	snprintf(tmps, 5, "%.f%%", round(fraction * 100));
@@ -860,21 +860,21 @@ void on_close_clicked(GtkButton *button, gpointer user_data) {
 			gtk_widget_hide(rsync_window);
 		}
 	}
-	if (cmdline_execute) on_main_destroy((GtkObject*) main_window, NULL);
+	if (cmdline_execute && !more && !cmdline_stayopen) on_main_destroy((GtkObject*) main_window, NULL);
 }
 
 
 void scroll_to_end(GtkTextView* view, gboolean final) {
-	GTimeVal curtime;
+	gint64 curtime;
 
 //	if (config_fastscroll && !final) g_get_current_time(&curtime);
-	g_get_current_time(&curtime);
-	if (!config_fastscroll || final || curtime.tv_sec - scroll_previous_run >= 1) {
+	curtime = g_get_monotonic_time() / 1000000;
+	if (!config_fastscroll || final || curtime - scroll_previous_run >= 1) {
 		GtkTextBuffer *buffer;
 		GtkTextIter iter;
 		GtkTextMark *mark;
 
-		scroll_previous_run = curtime.tv_sec;
+		scroll_previous_run = curtime;
 		buffer = gtk_text_view_get_buffer(view);
 		gtk_text_buffer_get_end_iter(buffer, &iter);
 		mark = gtk_text_buffer_create_mark(buffer, "end_mark", &iter, TRUE);
@@ -904,7 +904,7 @@ void rsync_cleanup(gpointer data) {
 			buffer = gtk_text_view_get_buffer(view);
 			gtk_text_buffer_insert_at_cursor(buffer, comm_after, -1);
 			scroll_to_end(view, FALSE);
-			if (config_log) fputs(comm_after, log_file);
+			if (log_file != NULL) fputs(comm_after, log_file);
 			g_free(comm_after);
 		}
 	}
@@ -968,7 +968,7 @@ gboolean out_watch(GIOChannel *source, GIOCondition condition, gpointer data) {
 
 		gtk_text_buffer_insert_at_cursor(buffer, (gchar*)str->str, -1);
 		scroll_to_end((GtkTextView*)data, FALSE);
-		if (config_log) fputs((gchar*)str->str, log_file);
+		if (log_file != NULL) fputs((gchar*)str->str, log_file);
 
 		if (!dryrunning) {
 			if (strchr(str->str, '%') && str->str[0] == ' ') {
@@ -1026,7 +1026,7 @@ gboolean err_watch(GIOChannel *source, GIOCondition condition, gpointer data) {
 		gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, (gchar*)str->str, -1, "fore-red", NULL);
 		scroll_to_end((GtkTextView*)data, FALSE);
 
-		if (config_log) fputs((gchar*)str->str, log_file);
+		if (log_file != NULL) fputs((gchar*)str->str, log_file);
 		tmpc = g_strconcat(error_list, str->str, NULL);
 		g_free(error_list);
 		error_list = tmpc;
@@ -1063,7 +1063,7 @@ void child_watch(GPid pid, gint status, gpointer data) {
 	}
 	scroll_to_end((GtkTextView*)data, FALSE);
 
-	if (config_log) fputs(str, log_file);
+	if (log_file != NULL) fputs(str, log_file);
 	g_free(str);
 	rsync_cleanup(NULL);
 	must_scroll = TRUE;
@@ -1137,9 +1137,9 @@ void on_rsync_show(GtkWidget *widget, gpointer user_data) {
 	if (is_set && !first) gtk_text_buffer_insert_at_cursor(buffer, "\n", -1);
 	else gtk_text_buffer_set_text(buffer, "", -1);
 	gtk_text_buffer_insert_at_cursor(buffer, comm_text, -1);
-	if (config_log) fputs(comm_text, log_file);
+	if (log_file != NULL) fputs(comm_text, log_file);
 	g_free(comm_text);
-	g_get_current_time(&startime);
+	startime = g_get_monotonic_time() / 1000000;
 
 	if (!halt && g_spawn_async_with_pipes(NULL, argv, NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &rsync_pid, NULL, &out, &err, NULL)) {
 		set_trayicon_icon(TRUE);
@@ -1359,19 +1359,18 @@ void on_button_switch_clicked(GtkButton *button, gpointer user_data) {
 
 void on_button_pause_clicked(GtkButton *button, gpointer user_data) {
 	GtkButton *tmpbutt = (GtkButton*) gtk_builder_get_object(builder, "button_pause");
-	GTimeVal tmptime;
+	gint64 tmptime;
 	if (paused) {
 		if (rsync_pid) superkill(SIGCONT);
 		gtk_button_set_label(tmpbutt, "gtk-media-pause");
 		set_rsync_title(groups[session_last], "running");
-		g_get_current_time(&tmptime);
-		startime.tv_sec += tmptime.tv_sec - pausedtime.tv_sec;
-		startime.tv_usec += tmptime.tv_usec - pausedtime.tv_usec;
+		tmptime = g_get_monotonic_time() / 1000000;
+		startime += tmptime - pausedtime;
 	} else {
 		if (rsync_pid) superkill(SIGSTOP);
 		gtk_button_set_label(tmpbutt, "gtk-media-play");
 		set_rsync_title(groups[session_last], "paused");
-		g_get_current_time(&pausedtime);
+		pausedtime = g_get_monotonic_time() / 1000000;
 		g_idle_add(scroll_idle, gtk_builder_get_object(builder, "textview_output"));
 	}
 	paused = !paused;
